package main

import (
	"fmt"
	"os"
	"slices"
	"strings"
	"time"

	log "github.com/sirupsen/logrus"

	"github.com/onyx-and-iris/q3rcon-proxy/pkg/udpproxy"
)

func main() {
	logLevel, err := getEnvInt("Q3RCON_LOGLEVEL")
	if err != nil {
		log.Fatalf("unable to parse Q3RCON_LEVEL: %s", err.Error())
	}
	if slices.Contains(log.AllLevels, log.Level(logLevel)) {
		log.SetLevel(log.Level(logLevel))
	}

	proxyHost := os.Getenv("Q3RCON_PROXY_HOST")
	if proxyHost == "" {
		proxyHost = "0.0.0.0"
	}

	targetHost := os.Getenv("Q3RCON_TARGET_HOST")
	if targetHost == "" {
		targetHost = "127.0.0.1"
	}

	proxies := os.Getenv("Q3RCON_TARGET_PORTS")
	if proxies == "" {
		log.Fatal("env Q3RCON_TARGET_PORTS required")
	}

	sessionTimeout, err := getEnvInt("Q3RCON_SESSION_TIMEOUT")
	if err != nil {
		log.Fatalf("unable to parse Q3RCON_SESSION_TIMEOUT: %s", err.Error())
	}
	if sessionTimeout == 0 {
		sessionTimeout = 20
	}

	for _, proxy := range strings.Split(proxies, ";") {
		go start(proxyHost, targetHost, proxy, sessionTimeout)
	}

	<-make(chan struct{})
}

func start(proxyHost, targetHost, ports string, sessionTimeout int) {
	proxyPort, targetPort := func() (string, string) {
		x := strings.Split(ports, ":")
		return x[0], x[1]
	}()

	hostAddr := fmt.Sprintf("%s:%s", proxyHost, proxyPort)
	proxyAddr := fmt.Sprintf("%s:%s", targetHost, targetPort)

	c, err := udpproxy.New(
		hostAddr, proxyAddr,
		udpproxy.WithSessionTimeout(time.Duration(sessionTimeout)*time.Minute))
	if err != nil {
		log.Fatal(err)
	}

	log.Printf("q3rcon-proxy initialized: [proxy] (%s) [target] (%s)", hostAddr, proxyAddr)

	log.Fatal(c.ListenAndServe())
}
